import { AdvEntity, SimpleAdvEntity } from "../types";
import { ValidationError } from "../utils/errors";
import { pool } from "../utils/db";
import { FieldPacket } from "mysql2";
import { v4 as uuid } from "uuid";

type AdvRecordResult = [AdvEntity[], FieldPacket[]];

export class AdvRecord implements AdvEntity {
  public id: string;
  public name: string;
  public description: string;
  public price: number;
  public url: string;
  public lat: number;
  public lon: number;
  constructor(obj: AdvEntity) {
    if (!obj.name || obj.name.length > 100) {
      throw new ValidationError(
        "Advertisement name cannot be empty and cannot have more than 100 signs"
      );
    }
    if (obj.description.length > 1000) {
      throw new ValidationError(
        "Advertisement description cannot have more than 1000 signs"
      );
    }
    if (obj.price < 0 || obj.price > 9999999) {
      throw new ValidationError(
        "Price cannot be be lower than 0 and higher than 9 999 999"
      );
    }

    //TODO check if URL is valid!?
    if (!obj.url || obj.url.length > 100) {
      throw new ValidationError(
        "Advertisement url cannot be empty and cannot have more than 100 signs"
      );
    }
    if (typeof obj.lat !== "number" || typeof obj.lon !== "number") {
      throw new ValidationError("Cannot find advertisement");
    }
    this.id = obj.id;
    this.name = obj.name;
    this.description = obj.description;
    this.price = obj.price;
    this.url = obj.url;
    this.lat = obj.lat;
    this.lon = obj.lon;
  }

  static async getOne(id: string): Promise<AdvRecord | null> {
    const [results] = (await pool.execute("SELECT * FROM `adv` WHERE id=:id", {
      id: id,
    })) as AdvRecordResult;

    return results.length === 0 ? null : new AdvRecord(results[0]);
  }

  static async findAll(name: string): Promise<SimpleAdvEntity[]> {
    const [results] = (await pool.execute(
      "SELECT * FROM `adv` WHERE `name` LIKE :search",
      {
        search: `%${name}%`,
      }
    )) as AdvRecordResult;

    return results.map((result) => {
      const { id, lat, lon } = result;
      return { id, lat, lon };
    });
  }

  async insert(): Promise<void> {
    if (!this.id) {
      this.id = uuid();
    } else {
      throw new Error("Cannot insert something what is already inserted!");
    }

    await pool.execute(
      "INSERT INTO `adv`(`id`, `name`, `description`, `price`, `url`, `lat`, `lon`) VALUES (:id, :name," +
        " :description, :price, :url, :lat, :lon)",
      this
    );
  }
}
